/*

Copyright (C) 2002-2003, Jeffrey D. Argast.

The authors make NO WARRANTY or representation, either express or implied,
with respect to this software, its quality, accuracy, merchantability, or
fitness for a particular purpose.  This software is provided "AS IS", and you,
its user, assume the entire risk as to its quality and accuracy.

Permission is hereby granted to use, copy, modify, and distribute this
software or portions thereof for any purpose, without fee, subject to these
conditions:

(1) If any part of the source code is distributed, then this
statement must be included, with this copyright and no-warranty notice
unaltered.

(2) Permission for use of this software is granted only if the user accepts
full responsibility for any undesirable consequences; the authors accept
NO LIABILITY for damages of any kind.

*/

#import "TaskTabView.h"
#import "MessageDefs.h"
#import "PerforceAction.h"

const int kNumIcons = 4;

@implementation TaskTabView

- (void) awakeFromNib
{
    fIcons = [[NSArray arrayWithObjects:[NSImage imageNamed:@"TaskProgressIcon1"],
									    [NSImage imageNamed:@"TaskProgressIcon2"],
									    [NSImage imageNamed:@"TaskProgressIcon3"],
									    [NSImage imageNamed:@"TaskProgressIcon4"],
									    nil] retain];
    	
    fIconSize = [[fIcons objectAtIndex:0] size];
    
	fAnimating = NO;
	
	fCurrentIcon = 0;
	
	NSNotificationCenter* nCenter = [NSNotificationCenter defaultCenter];

	[nCenter addObserver:self selector:@selector(handleActionListChanged:) name:kActionListChanged object:nil];

	fTimer = nil;
}

- (void) dealloc
{
    [fIcons release];
	[fTimer release];
	
	[super dealloc];
}

- (void) invalView
{
	NSView* tabView = [self tabView];
	
	NSRect tabFrame = [tabView bounds];
	
	tabFrame.size.width = 19;
	
	[tabView displayRect:tabFrame];
}

- (void) handleActionListChanged:(NSNotification*)aNotification
{
	NSArray* actionList = [PerforceAction runningActions];
	
	int num = [actionList count];
	
	if ( num > 0 )
	{
		if ( !fTimer )
		{
			fTimer = [[NSTimer scheduledTimerWithTimeInterval:0.1 target:self selector:@selector(timerTickle:) userInfo:nil repeats:YES] retain];
	
			fAnimating = YES;
			
			[self invalView];
		}
	}
	else
	{
		if ( fTimer )
		{
			[fTimer invalidate];
			
			[fTimer release];
			
			fTimer = nil;

			fAnimating = NO;
			
			[self invalView];
		}
	}
	
}

- (void) timerTickle:(id)sender
{
	fCurrentIcon += 1;
	
	fCurrentIcon = fCurrentIcon % kNumIcons;

	[self invalView];
}

- (NSSize) sizeOfLabel: (BOOL) computeMin
{
	fTextSize = [super sizeOfLabel:computeMin];
	
	NSSize labelSize = fTextSize;

	labelSize.width += (fIconSize.width + 2);
	
	return labelSize;
}

- (void) drawLabel: (BOOL) shouldTruncateLabel 
         inRect: (NSRect) labelRect
{
	[super drawLabel:shouldTruncateLabel inRect:labelRect];
	
	if ( fAnimating )
	{
		NSPoint p = labelRect.origin;
    
		p.x += (fTextSize.width + 2);
		
		[[fIcons objectAtIndex:fCurrentIcon] compositeToPoint:p operation:NSCompositePlusDarker];
	}
}

@end
